/*  Project bardemo
    Bernd Herd Software Entwicklung
    Copyright  1995. All Rights Reserved.

    SUBSYSTEM:    bardemo.exe Application
    FILE:         brdmmdic.cpp
    AUTHOR:       Bernd Herd


    OVERVIEW
    ========
    Source file for implementation of bardemoMDIClient (TMDIClient).      
*/


#include <owl\owlpch.h>
#pragma hdrstop


#include "bardmapp.h"
#include "brdmmdi1.h"
#include "brdmmdic.h"
#include "apxprint.h"
#include "apxprev.h"


//{{bardemoMDIClient Implementation}}


//
// Build a response table for all messages/commands handled
// by bardemoMDIClient derived from TMDIClient.
//
DEFINE_RESPONSE_TABLE1(bardemoMDIClient, TMDIClient)
//{{bardemoMDIClientRSP_TBL_BEGIN}}
    EV_COMMAND(CM_FILEPRINT, CmFilePrint),
    EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
    EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
    EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
    EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
    EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
//{{bardemoMDIClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ===========
// Construction/Destruction handling.
bardemoMDIClient::bardemoMDIClient (TModule* module)
 : TMDIClient (module)
{
    // Change the window's background color
    SetBkgndColor(RGB(0xff, 0xff, 0xe8));

    ChildCount = 0;

    // INSERT>> Your constructor code here.

}


bardemoMDIClient::~bardemoMDIClient ()
{
    Destroy();

    // INSERT>> Your destructor code here.

}


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ===========
// MDIClient site initialization.
void bardemoMDIClient::SetupWindow ()
{
    // Default SetUpWindow processing.
    TMDIClient::SetupWindow ();

}


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ==========
// Menu File Print command
void bardemoMDIClient::CmFilePrint ()
{
    //
    // Create Printer object if not already created.
    // 
    bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        //
        // Create Printout window and set characteristics.
        //
        APXPrintOut printout(theApp->Printer, Title, GetActiveMDIChild()->GetClientWindow(), true);

        theApp->Printing++;

        //
        // Bring up the Print dialog and print the document.
        //
        theApp->Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

        theApp->Printing--;
    }
}


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ==========
// Menu File Print Setup command
void bardemoMDIClient::CmFilePrintSetup ()
{
    bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        //
        // Bring up the Print Setup dialog.
        //
        theApp->Printer->Setup(this);
    }
}


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ==========
// Menu File Print Preview command
void bardemoMDIClient::CmFilePrintPreview ()
{
    bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
    if (theApp) {
        if (!theApp->Printer)
            theApp->Printer = new TPrinter(GetApplication());

        theApp->Printing++;

        PreviewWindow *prevW = new PreviewWindow(Parent, theApp->Printer, GetActiveMDIChild()->GetClientWindow(), "Print Preview", new TLayoutWindow(0));
        prevW->Create();

        GetApplication()->BeginModal(GetApplication()->GetMainWindow());

        // We must destroy the preview window explicitly.  Otherwise, the window will not be destroyed until
        // it's parent the MainWindow is destroyed.
        prevW->Destroy();
        delete prevW;

        theApp->Printing--;
    }
}


//////////////////////////////////////////////////////////
// bardemoMDIClient
// ==========
// Menu enabler used by Print, Print Setup and Print Preview.
void bardemoMDIClient::CmPrintEnable (TCommandEnabler &tce)
{
    if (GetActiveMDIChild()) {
        bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
        if (theApp) {
            // If we have a Printer already created just test if all is okay.
            // Otherwise create a Printer object and make sure the printer
            // really exists and then delete the Printer object.
            if (!theApp->Printer) {
                theApp->Printer = new TPrinter(GetApplication());
                
                tce.Enable(theApp->Printer->GetSetup().Error == 0);
            } else
                tce.Enable(theApp->Printer->GetSetup().Error == 0);
        }
    } else
        tce.Enable(false);
}


