/*  Project bardemo
    Bernd Herd Software Entwicklung
    Copyright  1995. All Rights Reserved.

    SUBSYSTEM:    bardemo.exe Application
    FILE:         brdmolvw.cpp
    AUTHOR:       Bernd Herd


    OVERVIEW
    ========
    Source file for implementation of bardemoOleView (TOleView).
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include "bardmapp.h"
#include "brdmolvw.h"

#include <stdio.h>

//{{bardemoOleView Implementation}}

//
// Build a response table for all messages/commands handled
// by bardemoOleView derived from TOleView.
//
DEFINE_RESPONSE_TABLE1(bardemoOleView, TOleView)
//{{bardemoOleViewRSP_TBL_BEGIN}}
    EV_WM_GETMINMAXINFO,
    EV_OC_VIEWSHOWTOOLS,
    EV_WM_SIZE,
    EV_EN_CHANGE(IDS_BAREDIT, EvEnChange),
    EV_WM_SETFOCUS,
//{{bardemoOleViewRSP_TBL_END}}
END_RESPONSE_TABLE;


//////////////////////////////////////////////////////////
// bardemoOleView
// ==========
// Construction/Destruction handling.
bardemoOleView::bardemoOleView (bardemoOleDocument& doc, TWindow* parent)
    : TOleView(doc, parent), BarDocument(doc)
{
    ToolBar = 0;

    // INSERT>> Your constructor code here.
    string InitString = BarDocument.GetCodeString();

    EditHeight = MulDiv(12, HIWORD(GetDialogBaseUnits()), 8) + 2*GetSystemMetrics(SM_CYBORDER);

    MyEdit  = new TMyEdit(this, IDS_BAREDIT, InitString.c_str(), -1, -1, 100, EditHeight);
}


void bardemoOleView::EvSize(uint sizeType, TSize& size)
{ MyEdit->SetWindowPos( NULL, 0, 0, size.cx, EditHeight, SWP_NOMOVE | SWP_NOZORDER);
  InvalidatePart(invView);

  TOleView::EvSize(sizeType, size);
}



// -------- User Changed Edit Contents -------------------
void bardemoOleView::EvEnChange()
{ bardemoOleDocument &Doc    = BarDocument;
  bardemoApp         *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
  TStatic            *TypeStatic;
  ECODE		      theType;

  char Buf[100];
  if (MyEdit->GetText(Buf, sizeof(Buf))) {
     string &OldString = Doc.GetCodeString();
     string  NewString(Buf);
     if (NewString != OldString) {
	Doc.SetCodeString(NewString);
	InvalidateRect(TRect(0,0,2000,1500), TRUE);
	InvalidatePart(invView);
//        TOleView::SetSelection(0);

	// ---- Transfer barcodename to static Element -----------
	TypeStatic = theApp->GetTypeStatic();
	if (TypeStatic) {
	   theType = BarCodeTyp( Buf, Doc.GetCodeTypes() );	// GET BT_xxxx Barcode Type Flag
	   TypeStatic->SetText( BarCodeName(theType) );
	}
     }
  }
}




void bardemoOleView::GetWindowClass(WNDCLASS far& wndclass)
{
  TOleView::GetWindowClass(wndclass);

  wndclass.style |= CS_HREDRAW | CS_VREDRAW;
}



void bardemoOleView::EvSetFocus(HWND hWndLostFocus)
{ TOleView::EvSetFocus(hWndLostFocus);
  MyEdit->SetFocus();
}


bardemoOleView::~bardemoOleView ()
{
    // INSERT>> Your destructor code here.

}


//
// Paint routine for Window, Printer, and PrintPreview for a TOleView client.
//
void bardemoOleView::Paint (TDC& dc, bool erase, TRect& rect)
{
    bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
    if (theApp) {
        // Only paint if we're printing and we have something to paint, otherwise do nothing.
        if (theApp->Printing && theApp->Printer && !rect.IsEmpty()) {
            // Use pageSize to get the size of the window to render into.  For a Window it's the client area,
            // for a printer it's the printer DC dimensions and for print preview it's the layout window.
            TSize   pageSize(rect.right - rect.left, rect.bottom - rect.top);

            TPrintDialog::TData &printerData = theApp->Printer->GetSetup();

            // Compute the number of pages to print.
            printerData.MinPage = 1;
            printerData.MaxPage = 1;

	    TOcView     *ocView = GetOcView();

	    //
            // Default TOcPart painting
            //
            TRect clientRect = GetClientRect();
	    TRect logicalRect = clientRect + (TSize&)ocView->GetOrigin();
    
            for (TOcPartCollectionIter i(GetOcDoc()->GetParts()); i; i++) {
    	        TOcPart& p = *i.Current();
    	        if (p.IsVisible(logicalRect)) {
    		        TRect r = p.GetRect();
			r -= ocView->GetOrigin();
    		        p.Draw(dc, r, clientRect);        // Draw the embedded object.
    		        if (p.IsSelected()) {
    			        TUIHandle handle(r,
    				                     TUIHandle::HandlesIn       |
    				                     TUIHandle::Grapples        |
						     TUIHandle::HatchBorder,
					 5);
				handle.Paint(dc);
			} else {
				TUIHandle handle(r, TUIHandle::HatchBorder, 5);
				handle.Paint(dc);
			}
		}
	    }

	    // INSERT>> Special printing code goes here.

	} else {
	    TOleView::Paint(dc, erase, rect);

	    // INSERT>> Normal painting code goes here.

	}

      { LOGFONT BarFont;
	int     Technology = dc.GetDeviceCaps(TECHNOLOGY);

	TRect clientRect = Technology == DT_RASDISPLAY ? GetClientRect()
//			   : Technology == DT_METAFILE   ? TRect(WindowOrg.x, WindowOrg.y, WindowExt.cx, WindowExt.cy)
						       : rect;

	// ------ Workaround for Problemswith OLE 1.x Containers ------
	if (Technology == DT_METAFILE) {
	    clientRect.right = 20000;
	    clientRect.bottom= 20000;

	    dc.SetMapMode(MM_ANISOTROPIC);
	    dc.SetWindowExt(TSize(clientRect.right, clientRect.bottom));
	}

/*	if (dc.GetDeviceCaps(TECHNOLOGY) != DT_RASDISPLAY) {
	   char x[200];
	   wsprintf(x, "(%d,%d,%d,%d)", clientRect.left,clientRect.top,clientRect.right,clientRect.bottom);
	   ::MessageBox(HWindow, x, "clientRect:", MB_OK);

	   MessageBeep(0);
	} */

	memset(&BarFont, 0, sizeof(BarFont));
	BarFont.lfHeight = 8;
	strcpy(BarFont.lfFaceName, "Arial");

	Bar2Hdc( HDC(dc),                        // Ausgabe-Device-Kontext
		 BarDocument.GetCodeString().c_str(),// Auszugebende BarCode-Nummer
		 &clientRect,                        // Ausgabe-Rechteck-Bereich
		 &BarFont,                       // Angaben fr zu verwendende Schriftart
		 BarDocument.GetCodeTypes()      // Maske der zulasigen BT_Xxxx-typen
		);
      }
    }
}

//
// Let container know about the server view size in pixels
//
bool bardemoOleView::EvOcViewPartSize(TOcPartSize far& ps)
{
  TClientDC dc(*this);

  TRect rect(0, 0, 0, 0);
    // a 2" x 2" extent for server
    //
    rect.right  = dc.GetDeviceCaps(LOGPIXELSX) * 2;
    rect.bottom = dc.GetDeviceCaps(LOGPIXELSY) * 2;

  ps.PartRect = rect;
  return true;
}

void bardemoOleView::EvGetMinMaxInfo (MINMAXINFO far& minmaxinfo)
{
    bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
    if (theApp) {
	if (theApp->Printing) {
	    minmaxinfo.ptMaxSize = TPoint(32000, 32000);
	    minmaxinfo.ptMaxTrackSize = TPoint(32000, 32000); 
	    return;
	}
    }
    TOleView::EvGetMinMaxInfo(minmaxinfo);
}


bool bardemoOleView::EvOcViewShowTools (TOcToolBarInfo far& tbi)
{
    // Construct & create a control bar for show, destroy our bar for hide
    if (tbi.Show) {
        if (!ToolBar) {
            ToolBar = new TControlBar(this);

            bardemoApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), bardemoApp);
            CHECK(theApp);

            theApp->CreateGadgets(ToolBar, true);
        }

	ToolBar->Create();
	tbi.HTopTB = (HWND)*ToolBar;
    } else {
	if (ToolBar) {
	    ToolBar->Destroy();
	    delete ToolBar;
	    ToolBar = 0;
	}
    }

    return true;
}




//
// Ask server to provide data according to the format in a handle
//
bool
bardemoOleView::EvOcViewClipData(TOcFormatData far& formatData)
{
  if (strcmp(OleStr(formatData.Format.GetRegName()), OleStr(BarVisionFormat)) != 0)
    return false; // not our clipboard format

  bool status = true;
  if (formatData.Paste) { // Pasting native data
    BarDocument.SetHandle(ofReadWrite, formatData.Handle, false);
    BarDocument.Open(ofRead, 0);
    Invalidate();
    // Restore the original storage
    //
    BarDocument.RestoreStorage();

  }
  else { // Copying native data
    HANDLE data = GlobalAlloc(GHND|GMEM_SHARE, 0);
    BarDocument.SetHandle(ofReadWrite, data, true);

    // Copy the selection if the target format is "DrawPad"
    //
/*    if (formatData.UserData) {
      status = BarDocument.CommitSelection(*this, formatData.UserData);
    }
    else {*/
      status = BarDocument.Commit(true);
//    }

    formatData.Handle = data;
    // Restore the original storage
    //
    BarDocument.RestoreStorage();
  }

  return status;
}




/* bardemoOleDocument ------------------------------------------*
 * Implementierung der Dokument-Klasse				*
 *--------------------------------------------------------------*/
bardemoOleDocument::bardemoOleDocument(TDocument* parent)
  : TOleDocument(parent)
{ BarCodeString = "023942380504";
  BarCodeTypes  = BT_NOADDON;
}

bardemoOleDocument::~bardemoOleDocument()
{
}



string bardemoOleDocument::GetCodeString(void)
{ return BarCodeString;
}

void bardemoOleDocument::SetCodeString(string AString)
{ BarCodeString=AString;
  SetDirty(true);
}



ECODE  bardemoOleDocument::GetCodeTypes(void)
{ return BarCodeTypes;
}

void bardemoOleDocument::SetCodeTypes(ECODE ABarCodeTypes)
{ BarCodeTypes = ABarCodeTypes;
  SetDirty(true);
}

bool
bardemoOleDocument::Commit(bool force)
{ TOleDocument::Commit(force);

  TOutStream* os = OutStream(ofWrite);
  if (!os)
    return false;

  // Append a description using a resource string
  *os <<       BarCodeString << '\n';
  *os << (int) BarCodeTypes;

  delete os;

  // Commit the storage if it was opened in transacted mode
  TOleDocument::CommitTransactedStorage();
  SetDirty(false);

  return true;
}


bool
bardemoOleDocument::Open(int mode, const char far* path)
{
  char linebuffer[100];

  TOleDocument::Open(mode, path);
  if (GetDocPath()) {
    TInStream* is = (TInStream*)InStream(ofRead);
    if (!is)
      return false;

    is->getline(linebuffer, sizeof(linebuffer));
    BarCodeString = linebuffer;

    *is >> (int) BarCodeTypes;

    delete is;
  } else
    BarCodeString = "Fehler?";

  SetDirty(false);
  return true;
}

bool
bardemoOleDocument::Close()
{
  return TOleDocument::Close();
}



static char* PropNames[] = {
  "BarCodeString",      // BarCodeString
  "BarCodeTypes" ,      // BarCodeTypes
};

static int PropFlags[] = {
  pfGetText, 		         // BarCodeString
  pfGetBinary | pfGetText,        // BarCodeTypes
};

const char*
bardemoOleDocument::PropertyName(int index)
{
  if (index <= PrevProperty)
    return TStorageDocument::PropertyName(index);  // OC server change
  else if (index < NextProperty)
    return PropNames[index-PrevProperty-1];
  else
    return 0;
}

int
bardemoOleDocument::PropertyFlags(int index)
{
  if (index <= PrevProperty)
    return TStorageDocument::PropertyFlags(index); // OC server change
  else if (index < NextProperty)
    return PropFlags[index-PrevProperty-1];
  else
    return 0;
}

int
bardemoOleDocument::FindProperty(const char far* name)
{
  for (int i=0; i < NextProperty-PrevProperty-1; i++)
    if (strcmp(PropNames[i], name) == 0)
      return i+PrevProperty+1;
  return 0;
}

int
bardemoOleDocument::GetProperty(int prop, void far* dest, int textlen)
{
  switch(prop) {
    case CodeTypes: {
      if (!textlen) {
	*(int far*)dest = BarCodeTypes;
	return sizeof(int);
      }
      return wsprintf((char far*)dest, "%d", (int) BarCodeTypes);
    }

    case BarCode:
      char* temp = new char[textlen]; // need local copy for medium model
      int len = BarCodeString.copy(temp, textlen);
      strcpy((char far*)dest, temp);
      return len;
  }
  return TStorageDocument::GetProperty(prop, dest, textlen); // OC server change
}





// -------- Construct UpperCase-Only Edit -----------------
TMyEdit::TMyEdit(TWindow* parent, int Id, const char far *text, int x, int y, int w, int h,
	 uint textLen, bool multiline, TModule* module) :
	 TEdit(parent, Id, text, x, y, w, h, textLen, multiline, module)
{ Attr.Style |= ES_UPPERCASE;
}


